@@ -1,8 +1,9 @@
 /*
  * Name:	rmccmt.c
- * Purpose:	Remove comments from C and C++ programs.
+ * Purpose:	Remove comments from C and C++ programs. Optionally removes
+ *  	    	code and leaves the comment instead.
  * Author:	Lars Wirzenius
- * Version:	"@(#)liwc:$Id: rmccmt.c,v 1.1.1.1 1996/09/16 18:19:51 liw Exp $"
+ * Version:	"@(#)liwc:$Id: rmccmt.c,v 1.3 2003/01/18 22:37:58 liw Exp $"
  */
 
 #include <stdio.h>
@@ -22,6 +23,19 @@ enum state {
 };
 
 static int keepnl;
+static int rmcode;
+
+static void print_cmt(int c)
+{
+    if (rmcode)
+    	putchar(c);
+}
+
+static void print_code(int c)
+{
+    if (!rmcode)
+    	putchar(c);
+}
 
 static int rmcmt(FILE *f, char *filename, void *dummy) {
 	enum state st;
@@ -37,14 +51,14 @@ static int rmcmt(FILE *f, char *filename, void *dummy) {
 				break;
 			case '"':
 				st = str;
-				putchar(c);
+				print_code(c);
 				break;
 			case '\'':
 				st = chr;
-				putchar(c);
+				print_code(c);
 				break;
 			default:
-				putchar(c);
+				print_code(c);
 				break;
 			}
 			break;
@@ -53,54 +67,65 @@ static int rmcmt(FILE *f, char *filename, void *dummy) {
 			switch (c) {
 			case '/':
 				st = incppcmt;
+				print_cmt('/');
+				print_cmt('/');
 				break;
 			case '*':
 				st = incmt;
+				print_cmt('/');
+				print_cmt('*');
 				break;
 			case '"':
 				st = str;
-				putchar('/');
-				putchar(c);
+				print_code('/');
+				print_code(c);
 				break;
 			case '\'':
 				st = chr;
-				putchar('/');
-				putchar(c);
+				print_code('/');
+				print_code(c);
 				break;
 			default:
-				putchar('/');
+				print_code('/');
 				st = code;
-				putchar(c);
+				print_code(c);
 				break;
 			}
 			break;
 
 		case incmt:
-			if (c == '\n' && keepnl)
+			print_cmt(c);
+			if (c == '\n' && keepnl && !rmcode)
 				putchar('\n');
 			if (c == '*')
 				st = endcmt;
 			break;
 
 		case endcmt:
-			if (c == '\n' && keepnl)
+			print_cmt(c);
+			if (c == '\n' && keepnl && !rmcode)
 				putchar('\n');
 			if (c == '/') {
 				st = code;
-				putchar(' '); /* replace cmt with space */
+				if (rmcode)
+				    putchar('\n'); /* next cmt on new line */
+				else
+				    putchar(' '); /* replace cmt with space */
 			} else if (c != '*')
 				st = incmt;
 			break;
 
 		case incppcmt:
+		    	print_cmt(c);
 			if (c == '\n') {
 				st = code;
-				putchar('\n');
+				if (!rmcode)
+				    putchar('\n');
 			}
 			break;
 
 		case str:
-			putchar(c);
+			print_code(c);
 			if (c == '\\')
 				st = strquote;
 			else if (c == '"')
@@ -108,12 +133,12 @@ static int rmcmt(FILE *f, char *filename, void *dummy) {
 			break;
 
 		case strquote:
-			putchar(c);
+			print_code(c);
 			st = str;
 			break;
 
 		case chr:
-			putchar(c);
+			print_code(c);
 			if (c == '\\')
 				st = chrquote;
 			else if (c == '\'')
@@ -121,7 +146,7 @@ static int rmcmt(FILE *f, char *filename, void *dummy) {
 			break;
 
 		case chrquote:
-			putchar(c);
+			print_code(c);
 			st = chr;
 			break;
 		}
@@ -142,8 +167,11 @@ static int rmcmt(FILE *f, char *filename, void *dummy) {
 int main(int argc, char **argv) {
 	set_progname(argv[0], "rmccmt");
 	
-	while (argc > 1 && strcmp(argv[1], "-n") == 0) {
-		keepnl = 1;
+	while (argc > 1 && argv[1][0] == '-') {
+	    	if (strcmp(argv[1], "-n") == 0)
+		    keepnl = 1;
+		else if (strcmp(argv[1], "-c") == 0)
+		    rmcode = 1;
 		--argc;
 		++argv;
 	}
